@@ -22,17 +22,17 @@
 #include <SDL.h>
 #include <SDL_thread.h>
 
-#include "dbg_types.h"
-#include "debugger.h"
-#include "dbg_breakpoints.h"
-#include "dbg_memory.h"
-
-#include "api/m64p_types.h"
 #include "api/callbacks.h"
+#include "api/m64p_types.h"
+#include "dbg_breakpoints.h"
+#include "dbg_debugger.h"
+#include "dbg_types.h"
+#include "memory/memory.h"
+
+#ifdef DBG
 
 int g_NumBreakpoints=0;
-breakpoint g_Breakpoints[BREAKPOINTS_MAX_NUMBER];
-
+m64p_breakpoint g_Breakpoints[BREAKPOINTS_MAX_NUMBER];
 
 int add_breakpoint( uint32 address )
 {
@@ -42,25 +42,24 @@ int add_breakpoint( uint32 address )
     }
     g_Breakpoints[g_NumBreakpoints].address=address;
     g_Breakpoints[g_NumBreakpoints].endaddr=address;
-    BPT_SET_FLAG(g_Breakpoints[g_NumBreakpoints], BPT_FLAG_EXEC);
+    BPT_SET_FLAG(g_Breakpoints[g_NumBreakpoints], M64P_BKP_FLAG_EXEC);
 
     enable_breakpoint(g_NumBreakpoints);
 
     return g_NumBreakpoints++;
 }
 
-int add_breakpoint_struct(breakpoint* newbp)
+int add_breakpoint_struct(m64p_breakpoint *newbp)
 {
      if( g_NumBreakpoints == BREAKPOINTS_MAX_NUMBER ) {
         DebugMessage(M64MSG_ERROR, "BREAKPOINTS_MAX_NUMBER have been reached.");
         return -1;
     }
 
-    memcpy(&g_Breakpoints[g_NumBreakpoints], newbp, sizeof(breakpoint));
+    memcpy(&g_Breakpoints[g_NumBreakpoints], newbp, sizeof(m64p_breakpoint));
 
-    if(BPT_CHECK_FLAG(g_Breakpoints[g_NumBreakpoints], BPT_FLAG_ENABLED))
-    {
-        BPT_CLEAR_FLAG(g_Breakpoints[g_NumBreakpoints], BPT_FLAG_ENABLED);
+    if (BPT_CHECK_FLAG(g_Breakpoints[g_NumBreakpoints], M64P_BKP_FLAG_ENABLED)) {
+        BPT_CLEAR_FLAG(g_Breakpoints[g_NumBreakpoints], M64P_BKP_FLAG_ENABLED);
         enable_breakpoint( g_NumBreakpoints );
     }
     
@@ -69,51 +68,51 @@ int add_breakpoint_struct(breakpoint* newbp)
 
 void enable_breakpoint( int bpt)
 {
-    breakpoint *curBpt = g_Breakpoints + bpt;
+    m64p_breakpoint *curBpt = g_Breakpoints + bpt;
     uint64 bptAddr;
     
-    if(BPT_CHECK_FLAG((*curBpt), BPT_FLAG_READ)) {
-        for(bptAddr = curBpt->address; bptAddr <= (curBpt->endaddr | 0xFFFF); bptAddr+=0x10000)
-            if(lookup_breakpoint((uint32) bptAddr & 0xFFFF0000, 0x10000, BPT_FLAG_ENABLED | BPT_FLAG_READ) == -1)
+    if (BPT_CHECK_FLAG((*curBpt), M64P_BKP_FLAG_READ)) {
+        for (bptAddr = curBpt->address; bptAddr <= (curBpt->endaddr | 0xFFFF); bptAddr+=0x10000)
+            if (lookup_breakpoint((uint32) bptAddr & 0xFFFF0000, 0x10000, M64P_BKP_FLAG_ENABLED | M64P_BKP_FLAG_READ) == -1)
                 activate_memory_break_read((uint32) bptAddr);
     }
 
-    if(BPT_CHECK_FLAG((*curBpt), BPT_FLAG_WRITE)) {
-        for(bptAddr = curBpt->address; bptAddr <= (curBpt->endaddr | 0xFFFF); bptAddr+=0x10000)
-            if(lookup_breakpoint((uint32) bptAddr & 0xFFFF0000, 0x10000, BPT_FLAG_ENABLED | BPT_FLAG_WRITE) == -1)
+    if (BPT_CHECK_FLAG((*curBpt), M64P_BKP_FLAG_WRITE)) {
+        for (bptAddr = curBpt->address; bptAddr <= (curBpt->endaddr | 0xFFFF); bptAddr+=0x10000)
+            if (lookup_breakpoint((uint32) bptAddr & 0xFFFF0000, 0x10000, M64P_BKP_FLAG_ENABLED | M64P_BKP_FLAG_WRITE) == -1)
                 activate_memory_break_write((uint32) bptAddr);
     }
     
-    BPT_SET_FLAG(g_Breakpoints[bpt], BPT_FLAG_ENABLED);
+    BPT_SET_FLAG(g_Breakpoints[bpt], M64P_BKP_FLAG_ENABLED);
 }
 
 void disable_breakpoint( int bpt )
 {
-    breakpoint *curBpt = g_Breakpoints + bpt;
+    m64p_breakpoint *curBpt = g_Breakpoints + bpt;
     uint64 bptAddr;
 
-    BPT_CLEAR_FLAG(g_Breakpoints[bpt], BPT_FLAG_ENABLED);
+    BPT_CLEAR_FLAG(g_Breakpoints[bpt], M64P_BKP_FLAG_ENABLED);
 
-    if(BPT_CHECK_FLAG((*curBpt), BPT_FLAG_READ)) {
-        for(bptAddr = curBpt->address; bptAddr <= ((unsigned long)(curBpt->endaddr | 0xFFFF)); bptAddr+=0x10000)
-            if(lookup_breakpoint((uint32) bptAddr & 0xFFFF0000, 0x10000, BPT_FLAG_ENABLED | BPT_FLAG_READ) == -1)
+    if (BPT_CHECK_FLAG((*curBpt), M64P_BKP_FLAG_READ)) {
+        for (bptAddr = curBpt->address; bptAddr <= ((unsigned long)(curBpt->endaddr | 0xFFFF)); bptAddr+=0x10000)
+            if (lookup_breakpoint((uint32) bptAddr & 0xFFFF0000, 0x10000, M64P_BKP_FLAG_ENABLED | M64P_BKP_FLAG_READ) == -1)
                 deactivate_memory_break_read((uint32) bptAddr);
     }
 
-    if(BPT_CHECK_FLAG((*curBpt), BPT_FLAG_WRITE)) {
-        for(bptAddr = curBpt->address; bptAddr <= ((unsigned long)(curBpt->endaddr | 0xFFFF)); bptAddr+=0x10000)
-            if(lookup_breakpoint((uint32) bptAddr & 0xFFFF0000, 0x10000, BPT_FLAG_ENABLED | BPT_FLAG_WRITE) == -1)
+    if (BPT_CHECK_FLAG((*curBpt), M64P_BKP_FLAG_WRITE)) {
+        for (bptAddr = curBpt->address; bptAddr <= ((unsigned long)(curBpt->endaddr | 0xFFFF)); bptAddr+=0x10000)
+            if (lookup_breakpoint((uint32) bptAddr & 0xFFFF0000, 0x10000, M64P_BKP_FLAG_ENABLED | M64P_BKP_FLAG_WRITE) == -1)
                 deactivate_memory_break_write((uint32) bptAddr);
     }
 
-    BPT_CLEAR_FLAG(g_Breakpoints[bpt], BPT_FLAG_ENABLED);
+    BPT_CLEAR_FLAG(g_Breakpoints[bpt], M64P_BKP_FLAG_ENABLED);
 }
 
 void remove_breakpoint_by_num( int bpt )
 {
     int curBpt;
     
-    if(BPT_CHECK_FLAG(g_Breakpoints[bpt], BPT_FLAG_ENABLED))
+    if (BPT_CHECK_FLAG(g_Breakpoints[bpt], M64P_BKP_FLAG_ENABLED))
         disable_breakpoint( bpt );
 
     for(curBpt=bpt+1; curBpt<g_NumBreakpoints; curBpt++)
@@ -133,18 +132,16 @@ void remove_breakpoint_by_address( uint32 address )
         remove_breakpoint_by_num( bpt );
 }
 
-void replace_breakpoint_num( int bpt, breakpoint* copyofnew )
+void replace_breakpoint_num( int bpt, m64p_breakpoint *copyofnew )
 {
-    
-    if(BPT_CHECK_FLAG(g_Breakpoints[bpt], BPT_FLAG_ENABLED))
-        disable_breakpoint( bpt );
+    if (BPT_CHECK_FLAG(g_Breakpoints[bpt], M64P_BKP_FLAG_ENABLED))
+        disable_breakpoint(bpt);
 
-    memcpy(&(g_Breakpoints[bpt]), copyofnew, sizeof(breakpoint));
+    memcpy(&g_Breakpoints[bpt], copyofnew, sizeof(m64p_breakpoint));
 
-    if(BPT_CHECK_FLAG(g_Breakpoints[bpt], BPT_FLAG_ENABLED))
-    {
-        BPT_CLEAR_FLAG(g_Breakpoints[bpt], BPT_FLAG_ENABLED);
-        enable_breakpoint( bpt );
+    if (BPT_CHECK_FLAG(g_Breakpoints[bpt], M64P_BKP_FLAG_ENABLED)) {
+        BPT_CLEAR_FLAG(g_Breakpoints[bpt], M64P_BKP_FLAG_ENABLED);
+        enable_breakpoint(bpt);
     }
 }
 
@@ -176,7 +173,7 @@ int lookup_breakpoint( uint32 address, uint32 size, uint32 flags)
 
 int check_breakpoints( uint32 address )
 {
-    return lookup_breakpoint( address, 1, BPT_FLAG_ENABLED | BPT_FLAG_EXEC );
+    return lookup_breakpoint(address, 1, M64P_BKP_FLAG_ENABLED | M64P_BKP_FLAG_EXEC);
 }
 
 
@@ -187,17 +184,15 @@ int check_breakpoints_on_mem_access( uint32 pc, uint32 address, uint32 size, uin
     //It automatically stops and updates the debugger on hit, so the memory access
     //functions only need to call it and can discard the result.
     int bpt;
-    if(run == 2)
-    {
-        bpt=lookup_breakpoint( address, size, flags );
-        if(bpt != -1)
-        {
-            if(BPT_CHECK_FLAG(g_Breakpoints[bpt], BPT_FLAG_LOG))
+    if (g_dbg_runstate == M64P_DBG_RUNSTATE_RUNNING) {
+        bpt = lookup_breakpoint(address, size, flags);
+        if (bpt != -1) {
+            if (BPT_CHECK_FLAG(g_Breakpoints[bpt], M64P_BKP_FLAG_LOG))
                 log_breakpoint(pc, flags, address);
-            
-            run = 0;
+
+            g_dbg_runstate = M64P_DBG_RUNSTATE_PAUSED;
             update_debugger(pc);
-        
+
             return bpt;
         }
     }
@@ -207,11 +202,14 @@ int check_breakpoints_on_mem_access( uint32 pc, uint32 address, uint32 size, uin
 int log_breakpoint(uint32 PC, uint32 Flag, uint32 Access)
 {
     char msg[32];
-    
-    if(Flag & BPT_FLAG_READ) sprintf(msg, "0x%08X read 0x%08X", PC, Access);
-    else if(Flag & BPT_FLAG_WRITE) sprintf(msg, "0x%08X wrote 0x%08X", PC, Access);
-    else sprintf(msg, "0x%08X executed", PC);
+    if (Flag & M64P_BKP_FLAG_READ)
+        sprintf(msg, "0x%08X read 0x%08X", PC, Access);
+    else if (Flag & M64P_BKP_FLAG_WRITE)
+        sprintf(msg, "0x%08X wrote 0x%08X", PC, Access);
+    else
+        sprintf(msg, "0x%08X executed", PC);
     DebugMessage(M64MSG_INFO, "BPT: %s", msg);
     return 0;
 }
 
+#endif
