@@ -20,25 +20,29 @@
  *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.          *
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
+#include <ctype.h>
+#include <stddef.h>
+#include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <ctype.h>
+
+#define __STDC_FORMAT_MACROS
+#include <inttypes.h>
 
 #define M64P_CORE_PROTOTYPES 1
-#include "api/m64p_types.h"
 #include "api/callbacks.h"
 #include "api/config.h"
 #include "api/m64p_config.h"
-
-#include "md5.h"
-#include "rom.h"
+#include "api/m64p_types.h"
 #include "main.h"
-#include "util.h"
-
+#include "md5.h"
 #include "memory/memory.h"
 #include "osal/preproc.h"
 #include "osd/osd.h"
+#include "r4300/r4300.h"
+#include "rom.h"
+#include "util.h"
 
 #define DEFAULT 16
 
@@ -49,9 +53,9 @@ static romdatabase_entry* ini_search_by_md5(md5_byte_t* md5);
 static _romdatabase g_romdatabase;
 
 /* Global loaded rom memory space. */
-unsigned char* rom = NULL;
+unsigned char* g_rom = NULL;
 /* Global loaded rom size. */
-int rom_size = 0;
+int g_rom_size = 0;
 
 unsigned char isGoldeneyeRom = 0;
 
@@ -59,62 +63,72 @@ m64p_rom_header   ROM_HEADER;
 rom_params        ROM_PARAMS;
 m64p_rom_settings ROM_SETTINGS;
 
-static m64p_system_type rom_country_code_to_system_type(unsigned short country_code);
+static m64p_system_type rom_country_code_to_system_type(uint16_t country_code);
 static int rom_system_type_to_ai_dac_rate(m64p_system_type system_type);
 static int rom_system_type_to_vi_limit(m64p_system_type system_type);
 
+static const uint8_t Z64_SIGNATURE[4] = { 0x80, 0x37, 0x12, 0x40 };
+static const uint8_t V64_SIGNATURE[4] = { 0x37, 0x80, 0x40, 0x12 };
+static const uint8_t N64_SIGNATURE[4] = { 0x40, 0x12, 0x37, 0x80 };
+
 /* Tests if a file is a valid N64 rom by checking the first 4 bytes. */
 static int is_valid_rom(const unsigned char *buffer)
 {
-    /* Test if rom is a native .z64 image with header 0x80371240. [ABCD] */
-    if((buffer[0]==0x80)&&(buffer[1]==0x37)&&(buffer[2]==0x12)&&(buffer[3]==0x40))
-        return 1;
-    /* Test if rom is a byteswapped .v64 image with header 0x37804012. [BADC] */
-    else if((buffer[0]==0x37)&&(buffer[1]==0x80)&&(buffer[2]==0x40)&&(buffer[3]==0x12))
-        return 1;
-    /* Test if rom is a wordswapped .n64 image with header  0x40123780. [DCBA] */
-    else if((buffer[0]==0x40)&&(buffer[1]==0x12)&&(buffer[2]==0x37)&&(buffer[3]==0x80))
+    if (memcmp(buffer, Z64_SIGNATURE, sizeof(Z64_SIGNATURE)) == 0
+     || memcmp(buffer, V64_SIGNATURE, sizeof(V64_SIGNATURE)) == 0
+     || memcmp(buffer, N64_SIGNATURE, sizeof(N64_SIGNATURE)) == 0)
         return 1;
     else
         return 0;
 }
 
-/* If rom is a .v64 or .n64 image, byteswap or wordswap loadlength amount of
- * rom data to native .z64 before forwarding. Makes sure that data extraction
- * and MD5ing routines always deal with a .z64 image.
+/* Copies the source block of memory to the destination block of memory while
+ * switching the endianness of .v64 and .n64 images to the .z64 format, which
+ * is native to the Nintendo 64. The data extraction routines and MD5 hashing
+ * function may only act on the .z64 big-endian format.
+ *
+ * IN: src: The source block of memory. This must be a valid Nintendo 64 ROM
+ *          image of 'len' bytes.
+ *     len: The length of the source and destination, in bytes.
+ * OUT: dst: The destination block of memory. This must be a valid buffer for
+ *           at least 'len' bytes.
+ *      imagetype: A pointer to a byte that gets updated with the value of
+ *                 V64IMAGE, N64IMAGE or Z64IMAGE according to the format of
+ *                 the source block. The value is undefined if 'src' does not
+ *                 represent a valid Nintendo 64 ROM image.
  */
-static void swap_rom(unsigned char* localrom, unsigned char* imagetype, int loadlength)
+static void swap_copy_rom(void* dst, const void* src, size_t len, unsigned char* imagetype)
 {
-    unsigned char temp;
-    int i;
+    if (memcmp(src, V64_SIGNATURE, sizeof(V64_SIGNATURE)) == 0)
+    {
+        size_t i;
+        const uint16_t* src16 = (const uint16_t*) src;
+        uint16_t* dst16 = (uint16_t*) dst;
 
-    /* Btyeswap if .v64 image. */
-    if(localrom[0]==0x37)
-        {
         *imagetype = V64IMAGE;
-        for (i = 0; i < loadlength; i+=2)
-            {
-            temp=localrom[i];
-            localrom[i]=localrom[i+1];
-            localrom[i+1]=temp;
-            }
-        }
-    /* Wordswap if .n64 image. */
-    else if(localrom[0]==0x40)
+        /* .v64 images have byte-swapped half-words (16-bit). */
+        for (i = 0; i < len; i += 2)
         {
-        *imagetype = N64IMAGE;
-        for (i = 0; i < loadlength; i+=4)
-            {
-            temp=localrom[i];
-            localrom[i]=localrom[i+3];
-            localrom[i+3]=temp;
-            temp=localrom[i+1];
-            localrom[i+1]=localrom[i+2];
-            localrom[i+2]=temp;
-            }
+            *dst16++ = m64p_swap16(*src16++);
         }
-    else
+    }
+    else if (memcmp(src, N64_SIGNATURE, sizeof(N64_SIGNATURE)) == 0)
+    {
+        size_t i;
+        const uint32_t* src32 = (const uint32_t*) src;
+        uint32_t* dst32 = (uint32_t*) dst;
+
+        *imagetype = N64IMAGE;
+        /* .n64 images have byte-swapped words (32-bit). */
+        for (i = 0; i < len; i += 4)
+        {
+            *dst32++ = m64p_swap32(*src32++);
+        }
+    }
+    else {
         *imagetype = Z64IMAGE;
+        memcpy(dst, src, len);
+    }
 }
 
 m64p_error open_rom(const unsigned char* romimage, unsigned int size)
@@ -127,7 +141,7 @@ m64p_error open_rom(const unsigned char* romimage, unsigned int size)
     int i;
 
     /* check input requirements */
-    if (rom != NULL)
+    if (g_rom != NULL)
     {
         DebugMessage(M64MSG_ERROR, "open_rom(): previous ROM image was not freed");
         return M64ERR_INTERNAL;
@@ -141,18 +155,17 @@ m64p_error open_rom(const unsigned char* romimage, unsigned int size)
     /* Clear Byte-swapped flag, since ROM is now deleted. */
     g_MemHasBeenBSwapped = 0;
     /* allocate new buffer for ROM and copy into this buffer */
-    rom_size = size;
-    rom = (unsigned char *) malloc(size);
-    if (rom == NULL)
+    g_rom_size = size;
+    g_rom = (unsigned char *) malloc(size);
+    if (g_rom == NULL)
         return M64ERR_NO_MEMORY;
-    memcpy(rom, romimage, size);
-    swap_rom(rom, &imagetype, rom_size);
+    swap_copy_rom(g_rom, romimage, size, &imagetype);
 
-    memcpy(&ROM_HEADER, rom, sizeof(m64p_rom_header));
+    memcpy(&ROM_HEADER, g_rom, sizeof(m64p_rom_header));
 
     /* Calculate MD5 hash  */
     md5_init(&state);
-    md5_append(&state, (const md5_byte_t*)rom, rom_size);
+    md5_append(&state, (const md5_byte_t*)g_rom, g_rom_size);
     md5_finish(&state, digest);
     for ( i = 0; i < 16; ++i )
         sprintf(buffer+i*2, "%02X", digest[i]);
@@ -163,6 +176,8 @@ m64p_error open_rom(const unsigned char* romimage, unsigned int size)
     ROM_PARAMS.systemtype = rom_country_code_to_system_type(ROM_HEADER.Country_code);
     ROM_PARAMS.vilimit = rom_system_type_to_vi_limit(ROM_PARAMS.systemtype);
     ROM_PARAMS.aidacrate = rom_system_type_to_ai_dac_rate(ROM_PARAMS.systemtype);
+    ROM_PARAMS.countperop = COUNT_PER_OP_DEFAULT;
+    ROM_PARAMS.cheats = NULL;
 
     memcpy(ROM_PARAMS.headername, ROM_HEADER.Name, 20);
     ROM_PARAMS.headername[20] = '\0';
@@ -178,6 +193,8 @@ m64p_error open_rom(const unsigned char* romimage, unsigned int size)
         ROM_SETTINGS.status = entry->status;
         ROM_SETTINGS.players = entry->players;
         ROM_SETTINGS.rumble = entry->rumble;
+        ROM_PARAMS.countperop = entry->countperop;
+        ROM_PARAMS.cheats = entry->cheats;
     }
     else
     {
@@ -187,6 +204,8 @@ m64p_error open_rom(const unsigned char* romimage, unsigned int size)
         ROM_SETTINGS.status = 0;
         ROM_SETTINGS.players = 0;
         ROM_SETTINGS.rumble = 0;
+        ROM_PARAMS.countperop = COUNT_PER_OP_DEFAULT;
+        ROM_PARAMS.cheats = NULL;
     }
 
     /* print out a bunch of info about the ROM */
@@ -194,19 +213,19 @@ m64p_error open_rom(const unsigned char* romimage, unsigned int size)
     DebugMessage(M64MSG_INFO, "Name: %s", ROM_HEADER.Name);
     imagestring(imagetype, buffer);
     DebugMessage(M64MSG_INFO, "MD5: %s", ROM_SETTINGS.MD5);
-    DebugMessage(M64MSG_INFO, "CRC: %x %x", sl(ROM_HEADER.CRC1), sl(ROM_HEADER.CRC2));
+    DebugMessage(M64MSG_INFO, "CRC: %" PRIX32 " %" PRIX32, sl(ROM_HEADER.CRC1), sl(ROM_HEADER.CRC2));
     DebugMessage(M64MSG_INFO, "Imagetype: %s", buffer);
-    DebugMessage(M64MSG_INFO, "Rom size: %d bytes (or %d Mb or %d Megabits)", rom_size, rom_size/1024/1024, rom_size/1024/1024*8);
-    DebugMessage(M64MSG_VERBOSE, "ClockRate = %x", sl(ROM_HEADER.ClockRate));
-    DebugMessage(M64MSG_INFO, "Version: %x", sl(ROM_HEADER.Release));
+    DebugMessage(M64MSG_INFO, "Rom size: %d bytes (or %d Mb or %d Megabits)", g_rom_size, g_rom_size/1024/1024, g_rom_size/1024/1024*8);
+    DebugMessage(M64MSG_VERBOSE, "ClockRate = %" PRIX32, sl(ROM_HEADER.ClockRate));
+    DebugMessage(M64MSG_INFO, "Version: %" PRIX32, sl(ROM_HEADER.Release));
     if(sl(ROM_HEADER.Manufacturer_ID) == 'N')
         DebugMessage(M64MSG_INFO, "Manufacturer: Nintendo");
     else
-        DebugMessage(M64MSG_INFO, "Manufacturer: %x", sl(ROM_HEADER.Manufacturer_ID));
-    DebugMessage(M64MSG_VERBOSE, "Cartridge_ID: %x", ROM_HEADER.Cartridge_ID);
+        DebugMessage(M64MSG_INFO, "Manufacturer: %" PRIX32, sl(ROM_HEADER.Manufacturer_ID));
+    DebugMessage(M64MSG_VERBOSE, "Cartridge_ID: %" PRIX16, ROM_HEADER.Cartridge_ID);
     countrycodestring(ROM_HEADER.Country_code, buffer);
     DebugMessage(M64MSG_INFO, "Country: %s", buffer);
-    DebugMessage(M64MSG_VERBOSE, "PC = %x", sl((unsigned int)ROM_HEADER.PC));
+    DebugMessage(M64MSG_VERBOSE, "PC = %" PRIX32, sl(ROM_HEADER.PC));
     DebugMessage(M64MSG_VERBOSE, "Save type: %d", ROM_SETTINGS.savetype);
 
     //Prepare Hack for GOLDENEYE
@@ -219,11 +238,11 @@ m64p_error open_rom(const unsigned char* romimage, unsigned int size)
 
 m64p_error close_rom(void)
 {
-    if (rom == NULL)
+    if (g_rom == NULL)
         return M64ERR_INVALID_STATE;
 
-    free(rom);
-    rom = NULL;
+    free(g_rom);
+    g_rom = NULL;
 
     /* Clear Byte-swapped flag, since ROM is now deleted. */
     g_MemHasBeenBSwapped = 0;
@@ -236,9 +255,9 @@ m64p_error close_rom(void)
 /* ROM utility functions */
 
 // Get the system type associated to a ROM country code.
-static m64p_system_type rom_country_code_to_system_type(unsigned short country_code)
+static m64p_system_type rom_country_code_to_system_type(uint16_t country_code)
 {
-    switch (country_code & 0xFF)
+    switch (country_code & UINT16_C(0xFF))
     {
         // PAL codes
         case 0x44:
@@ -290,6 +309,102 @@ static int rom_system_type_to_ai_dac_rate(m64p_system_type system_type)
     }
 }
 
+static size_t romdatabase_resolve_round(void)
+{
+    romdatabase_search *entry;
+    romdatabase_entry *ref;
+    size_t skipped = 0;
+
+    /* Resolve RefMD5 references */
+    for (entry = g_romdatabase.list; entry; entry = entry->next_entry) {
+        if (!entry->entry.refmd5)
+            continue;
+
+        ref = ini_search_by_md5(entry->entry.refmd5);
+        if (!ref) {
+            DebugMessage(M64MSG_WARNING, "ROM Database: Error solving RefMD5s");
+            continue;
+        }
+
+        /* entry is not yet resolved, skip for now */
+        if (ref->refmd5) {
+            skipped++;
+            continue;
+        }
+
+        if (!isset_bitmask(entry->entry.set_flags, ROMDATABASE_ENTRY_GOODNAME) &&
+            isset_bitmask(ref->set_flags, ROMDATABASE_ENTRY_GOODNAME)) {
+            entry->entry.goodname = strdup(ref->goodname);
+            if (entry->entry.goodname)
+                entry->entry.set_flags |= ROMDATABASE_ENTRY_GOODNAME;
+        }
+
+        if (!isset_bitmask(entry->entry.set_flags, ROMDATABASE_ENTRY_CRC) &&
+            isset_bitmask(ref->set_flags, ROMDATABASE_ENTRY_CRC)) {
+            entry->entry.crc1 = ref->crc1;
+            entry->entry.crc2 = ref->crc2;
+            entry->entry.set_flags |= ROMDATABASE_ENTRY_CRC;
+        }
+
+        if (!isset_bitmask(entry->entry.set_flags, ROMDATABASE_ENTRY_STATUS) &&
+            isset_bitmask(ref->set_flags, ROMDATABASE_ENTRY_STATUS)) {
+            entry->entry.status = ref->status;
+            entry->entry.set_flags |= ROMDATABASE_ENTRY_STATUS;
+        }
+
+        if (!isset_bitmask(entry->entry.set_flags, ROMDATABASE_ENTRY_SAVETYPE) &&
+            isset_bitmask(ref->set_flags, ROMDATABASE_ENTRY_SAVETYPE)) {
+            entry->entry.savetype = ref->savetype;
+            entry->entry.set_flags |= ROMDATABASE_ENTRY_SAVETYPE;
+        }
+
+        if (!isset_bitmask(entry->entry.set_flags, ROMDATABASE_ENTRY_PLAYERS) &&
+            isset_bitmask(ref->set_flags, ROMDATABASE_ENTRY_PLAYERS)) {
+            entry->entry.players = ref->players;
+            entry->entry.set_flags |= ROMDATABASE_ENTRY_PLAYERS;
+        }
+
+        if (!isset_bitmask(entry->entry.set_flags, ROMDATABASE_ENTRY_RUMBLE) &&
+            isset_bitmask(ref->set_flags, ROMDATABASE_ENTRY_RUMBLE)) {
+            entry->entry.rumble = ref->rumble;
+            entry->entry.set_flags |= ROMDATABASE_ENTRY_RUMBLE;
+        }
+
+        if (!isset_bitmask(entry->entry.set_flags, ROMDATABASE_ENTRY_COUNTEROP) &&
+            isset_bitmask(ref->set_flags, ROMDATABASE_ENTRY_COUNTEROP)) {
+            entry->entry.countperop = ref->countperop;
+            entry->entry.set_flags |= ROMDATABASE_ENTRY_COUNTEROP;
+        }
+
+        if (!isset_bitmask(entry->entry.set_flags, ROMDATABASE_ENTRY_CHEATS) &&
+            isset_bitmask(ref->set_flags, ROMDATABASE_ENTRY_CHEATS)) {
+            if (ref->cheats)
+                entry->entry.cheats = strdup(ref->cheats);
+            entry->entry.set_flags |= ROMDATABASE_ENTRY_CHEATS;
+        }
+
+        free(entry->entry.refmd5);
+        entry->entry.refmd5 = NULL;
+    }
+
+    return skipped;
+}
+
+static void romdatabase_resolve(void)
+{
+    size_t last_skipped = (size_t)~0ULL;
+    size_t skipped;
+
+    do {
+        skipped = romdatabase_resolve_round();
+        if (skipped == last_skipped) {
+            DebugMessage(M64MSG_ERROR, "Unable to resolve rom database entries (loop)");
+            break;
+        }
+        last_skipped = skipped;
+    } while (skipped > 0);
+}
+
 /********************************************************************************************/
 /* INI Rom database functions */
 
@@ -355,6 +470,9 @@ void romdatabase_open(void)
             search->entry.savetype = DEFAULT;
             search->entry.players = DEFAULT;
             search->entry.rumble = DEFAULT; 
+            search->entry.countperop = COUNT_PER_OP_DEFAULT;
+            search->entry.cheats = NULL;
+            search->entry.set_flags = ROMDATABASE_ENTRY_NONE;
 
             search->next_entry = NULL;
             search->next_crc = NULL;
@@ -376,6 +494,7 @@ void romdatabase_open(void)
             if(!strcmp(l.name, "GoodName"))
             {
                 search->entry.goodname = strdup(l.value);
+                search->entry.set_flags |= ROMDATABASE_ENTRY_GOODNAME;
             }
             else if(!strcmp(l.name, "CRC"))
             {
@@ -387,6 +506,7 @@ void romdatabase_open(void)
                     index = search->entry.crc1 >> 24;
                     search->next_crc = g_romdatabase.crc_lists[index];
                     g_romdatabase.crc_lists[index] = search;
+                    search->entry.set_flags |= ROMDATABASE_ENTRY_CRC;
                 }
                 else
                 {
@@ -407,43 +527,96 @@ void romdatabase_open(void)
             }
             else if(!strcmp(l.name, "SaveType"))
             {
-                if(!strcmp(l.value, "Eeprom 4KB"))
+                if(!strcmp(l.value, "Eeprom 4KB")) {
                     search->entry.savetype = EEPROM_4KB;
-                else if(!strcmp(l.value, "Eeprom 16KB"))
+                    search->entry.set_flags |= ROMDATABASE_ENTRY_SAVETYPE;
+                } else if(!strcmp(l.value, "Eeprom 16KB")) {
                     search->entry.savetype = EEPROM_16KB;
-                else if(!strcmp(l.value, "SRAM"))
+                    search->entry.set_flags |= ROMDATABASE_ENTRY_SAVETYPE;
+                } else if(!strcmp(l.value, "SRAM")) {
                     search->entry.savetype = SRAM;
-                else if(!strcmp(l.value, "Flash RAM"))
+                    search->entry.set_flags |= ROMDATABASE_ENTRY_SAVETYPE;
+                } else if(!strcmp(l.value, "Flash RAM")) {
                     search->entry.savetype = FLASH_RAM;
-                else if(!strcmp(l.value, "Controller Pack"))
+                    search->entry.set_flags |= ROMDATABASE_ENTRY_SAVETYPE;
+                } else if(!strcmp(l.value, "Controller Pack")) {
                     search->entry.savetype = CONTROLLER_PACK;
-                else if(!strcmp(l.value, "None"))
+                    search->entry.set_flags |= ROMDATABASE_ENTRY_SAVETYPE;
+                } else if(!strcmp(l.value, "None")) {
                     search->entry.savetype = NONE;
-                else
+                    search->entry.set_flags |= ROMDATABASE_ENTRY_SAVETYPE;
+                } else {
                     DebugMessage(M64MSG_WARNING, "ROM Database: Invalid save type on line %i", lineno);
+                }
             }
             else if(!strcmp(l.name, "Status"))
             {
-                if (string_to_int(l.value, &value) && value >= 0 && value < 6)
+                if (string_to_int(l.value, &value) && value >= 0 && value < 6) {
                     search->entry.status = value;
-                else
+                    search->entry.set_flags |= ROMDATABASE_ENTRY_STATUS;
+                } else {
                     DebugMessage(M64MSG_WARNING, "ROM Database: Invalid status on line %i", lineno);
+                }
             }
             else if(!strcmp(l.name, "Players"))
             {
-                if (string_to_int(l.value, &value) && value >= 0 && value < 8)
+                if (string_to_int(l.value, &value) && value >= 0 && value < 8) {
                     search->entry.players = value;
-                else
+                    search->entry.set_flags |= ROMDATABASE_ENTRY_PLAYERS;
+                } else {
                     DebugMessage(M64MSG_WARNING, "ROM Database: Invalid player count on line %i", lineno);
+                }
             }
             else if(!strcmp(l.name, "Rumble"))
             {
-                if(!strcmp(l.value, "Yes"))
+                if(!strcmp(l.value, "Yes")) {
                     search->entry.rumble = 1;
-                else if(!strcmp(l.value, "No"))
+                    search->entry.set_flags |= ROMDATABASE_ENTRY_RUMBLE;
+                } else if(!strcmp(l.value, "No")) {
                     search->entry.rumble = 0;
-                else
+                    search->entry.set_flags |= ROMDATABASE_ENTRY_RUMBLE;
+                } else {
                     DebugMessage(M64MSG_WARNING, "ROM Database: Invalid rumble string on line %i", lineno);
+                }
+            }
+            else if(!strcmp(l.name, "CountPerOp"))
+            {
+                if (string_to_int(l.value, &value) && value > 0 && value <= 4) {
+                    search->entry.countperop = value;
+                    search->entry.set_flags |= ROMDATABASE_ENTRY_COUNTEROP;
+                } else {
+                    DebugMessage(M64MSG_WARNING, "ROM Database: Invalid CountPerOp on line %i", lineno);
+                }
+            }
+            else if(!strncmp(l.name, "Cheat", 5))
+            {
+                size_t len1 = 0, len2 = 0;
+                char *newcheat;
+
+                if (search->entry.cheats)
+                    len1 = strlen(search->entry.cheats);
+                if (l.value)
+                    len2 = strlen(l.value);
+
+                /* initial cheat */
+                if (len1 == 0 && len2 > 0)
+                    search->entry.cheats = strdup(l.value);
+
+                /* append cheat */
+                if (len1 != 0 && len2 > 0) {
+                    newcheat = malloc(len1 + 1 + len2 + 1);
+                    if (!newcheat) {
+                        DebugMessage(M64MSG_WARNING, "ROM Database: Failed to append cheat");
+                    } else {
+                        strcpy(newcheat, search->entry.cheats);
+                        strcat(newcheat, ";");
+                        strcat(newcheat, l.value);
+                        free(search->entry.cheats);
+                        search->entry.cheats = newcheat;
+                    }
+                }
+
+                search->entry.set_flags |= ROMDATABASE_ENTRY_CHEATS;
             }
             else
             {
@@ -456,28 +629,7 @@ void romdatabase_open(void)
     }
 
     fclose(fPtr);
-
-    /* Resolve RefMD5 references */
-    for (search = g_romdatabase.list; search != NULL; search = search->next_entry)
-    {
-        if (search->entry.refmd5 != NULL)
-        {
-            romdatabase_entry *ref = ini_search_by_md5(search->entry.refmd5);
-            if (ref != NULL)
-            {
-                if(ref->savetype!=DEFAULT)
-                    search->entry.savetype = ref->savetype;
-                if(ref->status!=0)
-                    search->entry.status = ref->status;
-                if(ref->players!=DEFAULT)
-                    search->entry.players = ref->players;
-                if(ref->rumble!=DEFAULT)
-                    search->entry.rumble = ref->rumble;
-            }
-            else
-                DebugMessage(M64MSG_WARNING, "ROM Database: Error solving RefMD5s");
-        }
-    }
+    romdatabase_resolve();
 }
 
 void romdatabase_close(void)
@@ -492,6 +644,7 @@ void romdatabase_close(void)
             free(g_romdatabase.list->entry.goodname);
         if(g_romdatabase.list->entry.refmd5)
             free(g_romdatabase.list->entry.refmd5);
+        free(g_romdatabase.list->entry.cheats);
         free(g_romdatabase.list);
         g_romdatabase.list = search;
         }
