@@ -27,7 +27,10 @@
 extern "C" {
 #endif
 
+#include <stddef.h>
+#include <stdint.h>
 #include <string.h>
+
 #include "osal/preproc.h"
 
 /**********************
@@ -128,7 +131,7 @@ void to_big_endian_buffer(void *buffer, size_t length, size_t count);
 /**********************
      GUI utilities
  **********************/
-void countrycodestring(unsigned short countrycode, char *string);
+void countrycodestring(uint16_t countrycode, char *string);
 void imagestring(unsigned char imagetype, char *string);
 
 /**********************
