@@ -19,9 +19,12 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-#include <iostream>
-#include <iomanip>
 #include <string.h>
+#include <cmath>
+#include <iomanip>
+#include <iostream>
+#include <utility>
+
 #include "OGLFT.h"
 
 int wstrlen(const wchar_t * s) 
@@ -1184,7 +1187,7 @@ namespace OGLFT
         GLubyte* inverse = new GLubyte[ bitmap.rows * width ];
         GLubyte* inverse_ptr = inverse;
 
-        for(int r=0; r<bitmap.rows; r++)
+        for(unsigned int r=0; r<bitmap.rows; r++)
         {
             GLubyte* bitmap_ptr = &bitmap.buffer[bitmap.pitch * (bitmap.rows - r - 1)];
 
