@@ -19,22 +19,28 @@
  *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.          *
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
+#include <SDL.h>
 // On-screen Display
 #include <SDL_opengl.h>
 #include <SDL_thread.h>
+#include <stdarg.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
 
 #include "OGLFT.h"
+#include "api/m64p_types.h"
 #include "osd.h"
 
 extern "C" {
     #define M64P_CORE_PROTOTYPES 1
-    #include "api/m64p_config.h"
-    #include "api/config.h"
     #include "api/callbacks.h"
+    #include "api/config.h"
+    #include "api/m64p_config.h"
     #include "api/m64p_vidext.h"
     #include "api/vidext.h"
-    #include "main/main.h"
     #include "main/list.h"
+    #include "main/main.h"
     #include "osal/files.h"
     #include "osal/preproc.h"
     #include "plugin/plugin.h"
@@ -275,7 +281,7 @@ void osd_exit(void)
 
     // delete message queue
     SDL_LockMutex(osd_list_lock);
-    list_for_each_entry_safe(msg, safe, &l_messageQueue, osd_message_t, list) {
+    list_for_each_entry_safe_t(msg, safe, &l_messageQueue, osd_message_t, list) {
         osd_remove_message(msg);
         if (!msg->user_managed)
             free(msg);
@@ -366,7 +372,7 @@ void osd_render()
         fCornerPos[i] = 0.5f * l_fLineHeight;
 
     SDL_LockMutex(osd_list_lock);
-    list_for_each_entry_safe(msg, safe, &l_messageQueue, osd_message_t, list) {
+    list_for_each_entry_safe_t(msg, safe, &l_messageQueue, osd_message_t, list) {
         // update message state
         if(msg->timeout[msg->state] != OSD_INFINITE_TIMEOUT &&
            ++msg->frames >= msg->timeout[msg->state])
@@ -590,7 +596,7 @@ static osd_message_t * osd_message_valid(osd_message_t *testmsg)
 
     if (!l_OsdInitialized || !testmsg) return NULL;
 
-    list_for_each_entry(msg, &l_messageQueue, osd_message_t, list) {
+    list_for_each_entry_t(msg, &l_messageQueue, osd_message_t, list) {
         if (msg == testmsg)
             return testmsg;
     }
