@@ -22,6 +22,9 @@
 #ifndef __OSD_H__
 #define __OSD_H__
 
+#include "main/list.h"
+#include "osal/preproc.h"
+
 /******************************************************************
    osd_corner
    0    1    2 |
@@ -76,6 +79,7 @@ typedef struct {
 #define OSD_INFINITE_TIMEOUT 0xffffffff
     unsigned int frames; // number of frames in this state
     int user_managed; // structure managed by caller and not to be freed by us
+    struct list_head list;
 } osd_message_t;
 
 enum { R, G, B }; // for referencing color array
@@ -84,16 +88,54 @@ enum { R, G, B }; // for referencing color array
 extern "C" {
 #endif
 
+#ifdef M64P_OSD
+
 void osd_init(int width, int height);
 void osd_exit(void);
 void osd_render(void);
 osd_message_t * osd_new_message(enum osd_corner, const char *, ...);
 void osd_update_message(osd_message_t *, const char *, ...);
 void osd_delete_message(osd_message_t *);
-void osd_message_set_corner(osd_message_t *, enum osd_corner);
 void osd_message_set_static(osd_message_t *);
 void osd_message_set_user_managed(osd_message_t *);
 
+#else
+
+static osal_inline void osd_init(int width, int height)
+{
+}
+
+static osal_inline void osd_exit(void)
+{
+}
+
+static osal_inline void osd_render(void)
+{
+}
+
+static osal_inline osd_message_t * osd_new_message(enum osd_corner eCorner, const char *fmt, ...)
+{
+	return NULL;
+}
+
+static osal_inline void osd_update_message(osd_message_t *msg, const char *fmt, ...)
+{
+}
+
+static osal_inline void osd_delete_message(osd_message_t *msg)
+{
+}
+
+static osal_inline void osd_message_set_static(osd_message_t *msg)
+{
+}
+
+static osal_inline void osd_message_set_user_managed(osd_message_t *msg)
+{
+}
+
+#endif
+
 #ifdef __cplusplus
 }
 #endif
