@@ -19,17 +19,19 @@
  *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.          *
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
-void compare_interupt(void);
-void gen_dp(void);
-void init_interupt(void);
+#ifndef M64P_R4300_INTERUPT_H
+#define M64P_R4300_INTERUPT_H
 
-extern int vi_field;
-extern unsigned int next_vi;
+#include <stdint.h>
+
+void init_interupt(void);
 
 // set to avoid savestates/reset if state may be inconsistent
 // (e.g. in the middle of an instruction)
 extern int interupt_unsafe_state;
 
+void raise_maskable_interrupt(uint32_t cause);
+
 void gen_interupt(void);
 void check_interupt(void);
 
@@ -55,3 +57,4 @@ void load_eventqueue_infos(char *buf);
 #define HW2_INT     0x200
 #define NMI_INT     0x400
 
+#endif /* M64P_R4300_INTERUPT_H */
