@@ -19,16 +19,17 @@
  *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.          *
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
-#ifndef ASSEMBLE_H
-#define ASSEMBLE_H
-
-#include "r4300/recomph.h"
-#include "api/callbacks.h"
-#include "osal/preproc.h"
+#ifndef M64P_R4300_ASSEMBLE_H
+#define M64P_R4300_ASSEMBLE_H
 
+#include <stdint.h>
 #include <stdlib.h>
 
-extern long long int reg[32];
+#include "api/callbacks.h"
+#include "osal/preproc.h"
+#include "r4300/recomph.h"
+
+extern int64_t reg[32];
 
 #define EAX 0
 #define ECX 1
@@ -59,6 +60,8 @@ extern long long int reg[32];
 
 extern int branch_taken;
 
+extern const uint16_t trunc_mode, round_mode, ceil_mode, floor_mode;
+
 void jump_start_rel8(void);
 void jump_end_rel8(void);
 void jump_start_rel32(void);
@@ -834,5 +837,5 @@ static osal_inline void ffree_fpreg(int fpreg)
    put8(0xC0 + fpreg);
 }
 
-#endif // ASSEMBLE_H
+#endif /* M64P_R4300_ASSEMBLE_H */
 
