@@ -23,10 +23,11 @@
 
 #include "assemble.h"
 #include "interpret.h"
-
-#include "r4300/recomph.h"
-#include "r4300/r4300.h"
+#include "r4300/cached_interp.h"
+#include "r4300/cp1_private.h"
 #include "r4300/ops.h"
+#include "r4300/r4300.h"
+#include "r4300/recomph.h"
 
 static void genbc1f_test(void)
 {
