@@ -1,5 +1,5 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
- *   Mupen64plus - assemble.c                                              *
+ *   Mupen64plus - assemble.h                                              *
  *   Mupen64Plus homepage: http://code.google.com/p/mupen64plus/           *
  *   Copyright (C) 2007 Richard Goedeken (Richard42)                       *
  *   Copyright (C) 2002 Hacktarux                                          *
@@ -20,15 +20,19 @@
  *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.          *
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
-#ifndef __ASSEMBLE_H__
-#define __ASSEMBLE_H__
-
-#include "r4300/recomph.h"
-#include "api/callbacks.h"
+#ifndef M64P_R4300_ASSEMBLE_H
+#define M64P_R4300_ASSEMBLE_H
 
+#include <stdint.h>
 #include <stdlib.h>
+#include <sys/types.h>
 
-extern long long int reg[32];
+#include "api/callbacks.h"
+#include "api/m64p_types.h"
+#include "osal/preproc.h"
+#include "r4300/recomph.h"
+
+extern int64_t reg[32];
 
 #define RAX 0
 #define RCX 1
@@ -68,6 +72,8 @@ extern long long int reg[32];
 
 extern int branch_taken;
 
+extern const uint16_t trunc_mode, round_mode, ceil_mode, floor_mode;
+
 void jump_start_rel8(void);
 void jump_end_rel8(void);
 void jump_start_rel32(void);
@@ -115,7 +121,7 @@ static inline int rel_r15_offset(void *dest, const char *op_name)
     if (llabs(rel_offset) > 0x7fffffff)
     {
         DebugMessage(M64MSG_ERROR, "Error: destination %p more than 2GB away from r15 base %p in %s()", dest, reg, op_name);
-        asm(" int $3; ");
+        OSAL_BREAKPOINT_INTERRUPT;
     }
 
     return (int) rel_offset;
@@ -1178,5 +1184,5 @@ static inline void ffree_fpreg(int fpreg)
    put8(0xC0 + fpreg);
 }
 
-#endif /* __ASSEMBLE_H__ */
+#endif /* M64P_R4300_ASSEMBLE_H */
 
