@@ -24,11 +24,16 @@
 
 #include "assemble.h"
 #include "interpret.h"
-
-#include "r4300/recomph.h"
-#include "r4300/r4300.h"
-#include "r4300/ops.h"
+#include "r4300/cp1_private.h"
 #include "r4300/macros.h"
+#include "r4300/ops.h"
+#include "r4300/r4300.h"
+#include "r4300/recomp.h"
+#include "r4300/recomph.h"
+
+#if defined(COUNT_INSTR)
+#include "r4300/instr_counters.h"
+#endif
 
 void genadd_s(void)
 {
