@@ -20,18 +20,23 @@
  *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.          *
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
+#include <stdint.h>
 #include <stdio.h>
 
 #include "assemble.h"
 #include "interpret.h"
-
-#include "r4300/recomph.h"
-#include "r4300/recomp.h"
-#include "r4300/r4300.h"
-#include "r4300/ops.h"
-#include "r4300/macros.h"
-
 #include "memory/memory.h"
+#include "r4300/cached_interp.h"
+#include "r4300/macros.h"
+#include "r4300/ops.h"
+#include "r4300/r4300.h"
+#include "r4300/recomp.h"
+#include "r4300/recomph.h"
+#include "r4300/x86_64/regcache.h"
+
+#if defined(COUNT_INSTR)
+#include "r4300/instr_counters.h"
+#endif
 
 static void genbltz_test(void)
 {
